/*********************************************************************
* Software License Agreement:
*
* The software supplied herewith by Microchip Technology Incorporated
* (the "Company") for its PICmicro Microcontroller is intended and
* supplied to you, the Company's customer, for use solely and
* exclusively on Microchip PICmicro Microcontroller products. The
* software is owned by the Company and/or its supplier, and is
* protected under applicable copyright laws. All rights are reserved.
* Any use in violation of the foregoing restrictions may subject the
* user to criminal sanctions under applicable laws, as well as to
* civil liability for the breach of the terms and conditions of this
* license.
*
* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*********************************************************************/

#include <pic.h>
#include <stdlib.h>
//#include <time.h>
#include <htc.h>

#ifndef _XTAL_FREQ
 // Unless already defined assume 4MHz system frequency
 // This definition is required to calibrate __delay_us() and __delay_ms()
  #define _XTAL_FREQ 32000000/*32768*/
#endif

//#define FOSC                     32000000UL //ori
#define FOSC                       32000000UL
// Timer0: 1:32 prescale
//                                 76543210
#define OPTION_REG_INIT          0b00000101
// bit 7 nWPUEN = 0
// bit 6 INTEDG = 0
// bit 5 TMR0CS = 0
// bit 4 TMR0SE = 0
// bit 3 PSA = 0
// bit 2 = 1 |
// bit 1 = 0 |> PS = 64
// bit 0 = 1 |

#define TIMER0_PRESCALE          64L

#include "GenericTypeDefs.h"
#include "lcd.h"
#include "rtcc.h"
#include "input.h"
#include "i2c.h"
#include "mcp9800.h"
#include "mchp_support.h"

//                     1111 1100 0000 0000
//                     5432 1098 7654 3210
//__CONFIG(0x3FE4); // 0011 1111 1110 0100
//__CONFIG(0x1EFF); // 0001 1110 1111 1111
__CONFIG(FCMEN_ON & IESO_ON & CLKOUTEN_OFF & BOREN_ON & CPD_OFF & CP_OFF & MCLRE_ON & PWRTE_OFF & WDTE_OFF & FOSC_INTOSC);
__CONFIG(LVP_OFF & /*BORV_19 &*/ STVREN_ON & PLLEN_OFF);

// ORI
// FCMEN_ON & IESO_ON & CLKOUTEN_OFF & BOREN_ON & CPD_OFF & CP_OFF & MCLRE_ON & PWRTE_OFF & WDTE_OFF & FOSC_INTOSC
// LVP_OFF & BORV_19 & STVREN_ON & PLLEN_OFF

int display_time(void);
int display_temp(void);
char display_int(int t);
char display_volt(int t);
int display_pot(int p);
BOOL TimeBaseManager(void);
void acquire_display_data(void);
void acquire_temperature(BOOL);
void acquire_time(void);
int Update_LCD(void);

volatile char          blink;
bit                    TMR0_time_tic;
bit                    TMR0_button_flag;
int                    secondsTic = 0;
volatile unsigned char timebase_10ms;
volatile unsigned char TMR0_button_held_timer;
volatile unsigned char TMR0_temperature_timer;
static int testje = 0;
int testjeCtr = 0;
int nobuttonPressedTime = 0;

typedef enum { MODE_TIME, MODE_TEMPERATURE, MODE_POT} mode_t;
typedef enum { INCR_MINUTES, INCR_TENS, INCR_HOURS } mode_t_incr;

int         pot_value;
time_t      time_out;
event_t     btn_event;
char        time_set;
mode_t      display_mode;
mode_t_incr time_incr_mode;
BOOL        am;
BOOL        time_dp;
BCD_TYPE    bcd_data;

int main(void)
{
  int i, rc;
  int lowPowerMode = 0;
  time_t      t;
  int ptFirstTime = 1;
  blink = 0;

  OPTION_REG = OPTION_REG_INIT;
  OSCCONbits.IRCF = /*14*/ 0x0e;
  SPLLEN = 1;

  // peripheral configuration
  PORTC = 0;
  PORTD = 0;
  PORTE = 0;
  TRISC2 = 0;
  TRISE0 = 0;
  TRISE1 = 0;
  TRISE2 = 0;
  TRISD1 = 0;
  TRISD5 = 0;
  TRISD6 = 0;

  ANSA1 = 1;
  ANSA3 = 1;
  ANSB3 = 1;
  //ANSB1 = 1;

  WPUB1 = 1;  // Weak pull-up on RB1.



  display_mode = MODE_TEMPERATURE;
  //time_set = 0;

  rtcc_init();
  lcd_init();
  input_init();

  // LED flash for diagnose.
  for (i = 0; i < 5; i++)
  {
    RD1 = 1;
    __delay_ms(100);
    RD1 = 0;
    __delay_ms(100);
  }

  i2c_init();
  t = time(0);



// Modified GVE
  LCDIE = 0;
//  LCDIE = 1;

  PEIE = 1;
  IOCIE = 1;
  IOCBP1 = 1;
  IOCBN1 = 1;
  GIE = 1;

  // External HW configuration.
  mcp9800_init();

  // Light all segments.
  B1 = G1 = C1 = 1;
  A1 = F1 = E1 = D1 = 1;
  B2 = G2 = C2 = DP1 = 1;
  D2 = E2 = F2 = A2 = 1;
  C3 = G3 = B3 = 1;
  D3 = E3 = F3 = A3 = 1;
  // wait for 2 seconds
  //for(time_out = t+2; time_out > time(0); );
  __delay_ms(1200);

  // Wait for LCD ready.
  while(!WA);

  // Clear all segments.
  B1 = G1 = C1 = 0;
  A1 = F1 = E1 = D1 = 0;
  B2 = G2 = C2 = DP1 = 0;
  D2 = E2 = F2 = A2 = 0;
  C3 = G3 = B3 = 0;
  D3 = E3 = F3 = A3 = 0;

  ////////////////////////////////////////// main loop ////////////////////////////////////////////////
  while (1)
  {

    rc = 0;
    if (nobuttonPressedTime > 1000) lowPowerMode = 1; else lowPowerMode = 0;
    if (lowPowerMode)
    {
      while (!rc) rc = Update_LCD();
      SLEEP();
      NOP();
    }


    if (!lowPowerMode) while (!TimeBaseManager());
    if (nobuttonPressedTime < 2000) nobuttonPressedTime++;

    if (display_mode == MODE_POT)
    {
      pot_value = input_pot();
      pot_value *= 2;
    }
    btn_event = input_event();

    switch(btn_event)
    {
      case BUTTON_UP:
        break;

      case BUTTON_DOWN:
        nobuttonPressedTime = 0;
        if (display_mode == MODE_TIME && time_set == 0)
        {
          if (TMR0_button_flag)
            if (TMR0_button_held_timer-- == 0)
            {
              // Enter time set mode when the button is held long enough.
              time_set = 1;
              // Time set routine will decrement TMR0_button_held_timer to zero
              // and make the first time adjustment immediately.
              TMR0_button_held_timer++;
              // read the current time into t and strip off seconds
              time(&t);
              t = t - (t % 60);
              time_incr_mode = INCR_MINUTES;
            }
            else
            {
              TMR0_button_flag = 0;
            }
        }
        break;

      case BUTTON_PRESSED:
        nobuttonPressedTime = 0;
        if (display_mode == MODE_TIME)
        { // Set the held-button timer for 0.5 sec (0.5/Timebase manager resolution).
          // Button must be held down for this amount of time to enter time set mode.
          TMR0_button_held_timer = 50;
        }
        break;

      case BUTTON_RELEASED: // On release, next display mode (unless we were setting the time).
        nobuttonPressedTime = 2000;
        if (time_set == 0)
        {
          switch (display_mode)
          {
            case MODE_TIME:
              TMR0_temperature_timer = 1;
              display_mode = MODE_TEMPERATURE;
              break;
            case MODE_TEMPERATURE:
              display_mode = MODE_POT;
              break;
            default:
            case MODE_POT:
              display_mode = MODE_TIME;
              break;
          }
        }
        else
        {
          // Exit time set mode on button release.
          // Time resumes from where it was left.
          time_set = 0;
        }
        break;

      default:
        break;
    }

    // Time set increments the clock time by one minute for every held button interval.
    // When the minutes reach an even 10 then the increment is 10 minutes.
    // When the minutes reach an even hour then the increment is one hour.
    if (time_set) // if we are setting the time ...
    {
      nobuttonPressedTime = 0;
      if (TMR0_time_tic)
      {
        TMR0_time_tic = 0;
        if (TMR0_button_held_timer-- == 0)
        {
          // increments occur every 1/3 second while the button is held
          TMR0_button_held_timer =  33;
          // adjust the time
          switch (time_incr_mode)
          {
            case INCR_MINUTES:
              t += 60;             // increment minutes while not even 10
              if ((t % 600)==0)    // check even 10
                time_incr_mode = INCR_TENS;
              break;
            case INCR_TENS:
              t += 600;             // increment by 10 minutes while not even hour
              if ((t % 3600)==0)    // check even 60
                time_incr_mode = INCR_HOURS;
              break;
            case INCR_HOURS:
            default:
              t += 3600;  // increment hour
              break;
          } // end - switch time_incr_mode
        } // end - button held timer
        // show the current setting
        rtcc_set(&t);
      } // end - TMR0_time_tic
    } // end - time set

    acquire_display_data();
    if (!lowPowerMode) rc = Update_LCD();

  } // end - main loop

  return 0;
}

void interrupt isr()
{
  static bit icon;

  // Timer 1
  if (TMR1IF && TMR1IE)
  {
    TMR1IF = 0;
    rtcc_handler();
    icon = !icon;
    secondsTic = 1;
    //S2 = ~S2;

    if(blink && RE0)
      lcd_display_off();
    else
      lcd_display_on();




  }

  if (LCDIF && LCDIE)
  {
    LCDIF = 0;
    //if (icon)
      ///*S2 = 1;*/ S2 = 0;
    //else
      //S2 = 0;
  }

  // Interrupt on change.
  if (IOCBF1 && IOCIE)
  {
    //PORTB;  // Read PORTB to clear mismatch condition.
    IOCBF1 = 0;
    //RD1 = 1;
    nobuttonPressedTime = 0;
  }

  i2c_handler();
}

int display_temp(void)
{
  int rc;

  // Temperature is assumed to have been acquired, converted, and stored in global bcd_data.

  rc = lcd_display_digits(bcd_data);
  if (bcd_data.digit3 == 3)
  {
    //BC4 = 0;
    DP1 = 1;
  }
  else if (bcd_data.digit3 == 4)
  {
    //BC4 = 0;
    DP1 = 0;
  }
  else
  {
    DP1 = 1;
  }

  return rc;
}

int display_pot(int p)
{
  int rc;

  rc = display_volt(p);
  //DP3 = 1;
  //DP2 = 0;
  //S1 = 0;
  ///*S2 = 1*/ S2 = 0;
  //DH = 1;
  //RH = 0;
  //RC = 0;

  return rc;
}

char display_volt(int t)
{
  BCD_TYPE bcd;

  bcd.digit0 = 17;
  t /= 10;
  bcd.digit1 = t % 10;
  t /= 10;
  bcd.digit2 = t % 10;
  t /= 10;
  bcd.digit3 = t % 10;
  return lcd_display_digits(bcd);
}

char display_int(int t)
{
  BCD_TYPE bcd;

  // assumes t is the temperature in degrees C * 10
  bcd.digit0 = t % 10;
  t /= 10;
  bcd.digit1 = t % 10;
  t /= 10;
  bcd.digit2 = t % 10;
  t /= 10;
  bcd.digit3 = t%10;
  return lcd_display_digits(bcd);
}

int display_time(void)
{
  int rc;

  // display time in 12 hour format
  // time is assumed to have been acquired, converted, and stored in global bcd_data
  //RD1 = 0;

  rc = lcd_display_digits(bcd_data);
  //if (time_dp)
  //  DP3 = 1;
  //else
  //  DP3 = 0;

  //if (am)
  //  AMPS = 1;
  //else
  //  AMPS = 0;

  //DP2 = 0;
  //S1 = 0;
  //AMPS = 0;
  //VOLT = 0;
  //KILO = 0;
  //OHMS = 0;
  ///*RC = 1*/RC = 0;
  //DH = 0;
  //RH = 0;

  //RD1 = 0;
  return rc;
}

//////////////////////////////////////// definitions for TimeBaseManager //////////////////////////////////////////

#define SYSTEM_FREQUENCY                  (FOSC/4L)
#define MILLISECONDS_PER_SEC              1000L

// number of milliseconds in each timebase count
// (i.e. resolution of TimebaseManager() in milliseconds)
#define TIMEBASE_MS_PER_COUNT             10L

// number of milliseconds in each timebase manager tick clock
#define TIMEBASE_MS_PER_TICK              2L
#define TIMEBASE_LOAD_10ms                (TIMEBASE_MS_PER_COUNT/TIMEBASE_MS_PER_TICK)

// timebase reload value for TimebaseManager() interrupt period:
#define TIMEBASE_MANAGER_PERIOD_COUNT     (TIMEBASE_MS_PER_TICK*SYSTEM_FREQUENCY)/(TIMER0_PRESCALE*MILLISECONDS_PER_SEC)

#define TIMEBASE_MANAGER_RELOAD_COUNT     (0x00FF - TIMEBASE_MANAGER_PERIOD_COUNT)

#if (TIMEBASE_MANAGER_PERIOD_COUNT > 0x00FF)
   #error "TIMER0_PRESCALE is too small. Timebase reload count exceeds 0xFF."
#endif

#if (TIMEBASE_MANAGER_PERIOD_COUNT < 0x80)
   #warning "TIMER0_PRESCALE can be decreased to improve timebase resolution"
#endif

BOOL TimeBaseManager(void)
{
  if (T0IF)
  {
    T0IF = 0;
    TMR0 += TIMEBASE_MANAGER_RELOAD_COUNT;

    if(timebase_10ms)
    {
      timebase_10ms--;
      return TRUE;        // wait 10ms
    }

    // 10ms TIMEBASE timer
    //--------------------------------------------------------------------------
    timebase_10ms = TIMEBASE_LOAD_10ms;

    // For regulating main loop time
    TMR0_time_tic = 1;
    // For timing held buttons
    TMR0_button_flag = 1;
    //RD1 = 0;

    return TRUE;
  }
  //RD1 = 0;
  return FALSE;
}

int Update_LCD(void)
{
  int rc;

  // call this routine upon sensing WA rising edge
  switch (display_mode)
  {
    case MODE_TIME:
      rc = display_time();
      break;
    case MODE_TEMPERATURE:
      rc = display_temp();
      break;
    case MODE_POT:
      rc = display_pot(pot_value);
      break;
    default:
      break;
  }

  return rc;
}

void acquire_time(void)
{
  // Acquire the present time and convert to BCD digits. Store result in global bcd_data.

  time_t t;
  struct tm *d;

  time(&t);
  d = gmtime(&t);

  // time_dp is blinking seconds indicator
  time_dp = ((t % 2) == 1);

  bcd_data.digit0 = d->tm_min % 10;
  bcd_data.digit1 = d->tm_min / 10;

  if(d->tm_hour > 11)
  {
    d->tm_hour -= 12;
    am = FALSE;  // PM indicator
  }
  else
  {
    am = TRUE;   // AM indicator
  }

  if(d->tm_hour == 0) d->tm_hour = 12;

  bcd_data.digit2 = d->tm_hour % 10;
  bcd_data.digit3 = d->tm_hour/10;
}

void acquire_temperature(BOOL ModeF)
{
  // Acquire temperature from remote device and convert and store in global bcd_data.
  int deg;

  deg = mcp9800_get_temp();
  //deg = 215;
  // TEST
  //deg = -72;
  // assumes t is the temperature in degrees C * 10
  if(ModeF) deg = ((9 * deg) / 5) + 320;

  // Example:
  //  temp = 34.8
  //  LCD         3    4 .  8
  //  digit  3    2    1    0
#define BASE 10
  if (deg >= 0)
  {
    bcd_data.digit0 = deg % BASE;
    deg /= BASE;
    bcd_data.digit1 = deg % BASE;
    deg /= BASE;
    bcd_data.digit2 = deg % BASE;
    deg /= BASE;
    bcd_data.digit3 = deg % BASE;

    // Suppress leading zeros.
    if (bcd_data.digit2 == 0 && bcd_data.digit3 == 0)
      bcd_data.digit2 = 18;
    return;
  }

  // Example:
  //  temp = -9.7
  //  LCD         -    9 .  7
  //  digit  3    2    1    0
  if (deg < 0 && deg > -100)
  {
    deg *= -1;
    bcd_data.digit0 = deg % 10;
    deg /= 10;
    bcd_data.digit1 = deg % 10;
    bcd_data.digit2 = 16;
    bcd_data.digit3 = 3;
    return;
  }

  // Example:
  //  temp = -12
  //  LCD         -    1    2
  //  digit  3    2    1    0
  if (deg <= -100)
  {
    deg *= -1;
    deg /= 10;
    bcd_data.digit0 = deg % 10;
    deg /= 10;
    bcd_data.digit1 = deg % 10;
    bcd_data.digit2 = 16;
    bcd_data.digit3 = 4;
    return;
  }

}

void acquire_display_data(void)
{
  // Acquire and convert selected display data.
  // Conversions are made at loop rate so data is
  // ready to be written in the short LCD WA window.

  switch(display_mode)
  {
    case MODE_TIME:
      acquire_time();
      break;

    case MODE_TEMPERATURE:
      // Flag is set every 10 ms by TimeBaseManager.
      if (TMR0_time_tic && FALSE)
      {
        TMR0_time_tic = 0;
        // Acquire new temperature every 5 sec.
        if (--TMR0_temperature_timer == 0)
        {
          acquire_temperature(FALSE);
          TMR0_temperature_timer = 500;
        }
      }
      acquire_temperature(FALSE);
      break;

    default:
      break;
  }
}
